//#define DEBUG //PB1 switch on timeroverflow und during valid IR reception
#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "data.h"
#include "IR.h"

#ifdef CAPTURE
//use the timer1 capture capability
#define TIMEVAL	ICR1  //the input capture reg
#define IRVECT	TIMER1_CAPT_vect
#else
//use the interrupt input for IR edges and capture manually (less accurate)
#define IRPORT	PINB
#define IRDIR	DDRB
#define IRBIT	PB0
#define IRVECT	INT1_vect
#define TIMEVAL	TCNT1	//the timer reg
#endif

/* based on 500ns timer resolution
#define PULSEW0L	1400
#define PULSEW0U	1700
#define PULSEW1L	1700
#define PULSEW1U	1900
#define PULSEW2L	1900
#define PULSEW2U	2100
#define PULSEW3L	2100
#define PULSEW3U	2400
*/
#if  F_CPU == 7372800
//based on 1085ns timer resolution
#define PULSEW0L	645
#define PULSEW0U	783
#define PULSEW1L	783
#define PULSEW1U	876
#define PULSEW2L	876
#define PULSEW2U	968
#define PULSEW3L	968
#define PULSEW3U	1106
#elif F_CPU == 14745600
//based on 542ns timer resolution
#define PULSEW0L	1290
#define PULSEW0U	1566
#define PULSEW1L	1566
#define PULSEW1U	1752
#define PULSEW2L	1752
#define PULSEW2U	1936
#define PULSEW3L	1936
#define PULSEW3U	2212
#else
#error "define divider for cpu freq "  F_CPU
#endif

typedef union
{ long asLong;
	struct//lowest bit first
	{ unsigned stat:2; //low order byte, low word
	  unsigned tail:1;
	  unsigned par:1;
	  unsigned mot2d:1;
	  unsigned mot1d:1;
	  unsigned mot4d:1;
	  unsigned mot3d:1;
	  unsigned mot2s:4; //high byte, low word
	  unsigned mot1s:4;
	  unsigned mot4s:4; //low byte high word
	  unsigned mot3s:4;
	  unsigned on:1;	//high byte high word
	  unsigned off:1;
	  unsigned freq:1;
	  unsigned rec:1;
	  unsigned head:4;
	} asStruct;
} IR_type;

#ifdef USE_IR

IR_type IR_code;
int overflow = 0;

void init_IR()
{
  TCCR1A = 0; //normal mode
#if  F_CPU == 7372800
  TCCR1B = _BV(ICNC1) | 2; //clk/8 ~ 900kHz ~ 1.1us
#elif F_CPU == 14745600
  TCCR1B = _BV(ICNC1) | 2; //clk/8 ~ 1800kHz ~ 0.55us
#else
#error "define divider for cpu freq "  F_CPU
#endif

#ifdef CAPTURE
//  TCCR1B |= _BV(ICES1); //select positive edge because analog comparator inverts
  TIMSK1 |= _BV(ICIE1) | _BV(TOIE1); //int on input capture and on overflow
//  ACSR   = _BV(ACD) | _BV(ACBG) | _BV(ACIC); //comp enable, bandgap, timer capture
#else
  IRDIR &= ~_BV(IRBIT); //set as input
  EICRA = (EICRA & ~_BV(ISC10)) | _BV(ISC11); //int1 falling edge
  EIMSK |= _BV(INT1); //enable int1
  TIMSK1 = _BV(TOIE1); //int on ovfl
#endif
}

void get_IR()
{ msg.body.cmd102.input.uni[0] = IR_code.asStruct.mot1d?IR_code.asStruct.mot1s:-IR_code.asStruct.mot1s;
  msg.body.cmd102.input.uni[1] = IR_code.asStruct.mot2d?-IR_code.asStruct.mot2s:IR_code.asStruct.mot2s;
  msg.body.cmd102.input.uni[2] = IR_code.asStruct.mot3d?IR_code.asStruct.mot3s:-IR_code.asStruct.mot3s;
  msg.body.cmd102.input.uni[3] = IR_code.asStruct.mot4d?-IR_code.asStruct.mot4s:IR_code.asStruct.mot4s;
  msg.body.cmd102.input.uni[4] = IR_code.asStruct.on;
  msg.body.cmd102.input.uni[5] = IR_code.asStruct.off;
  msg.body.cmd102.input.uni[6] = IR_code.asStruct.freq;
  msg.body.cmd102.input.uni[7] = IR_code.asStruct.rec;
}

//timer (upcounter) to measure length of IR pulses
ISR(TIMER1_OVF_vect)
{ 	SETPIN11;
    if (overflow < 5) 
		overflow++;
	else if (overflow==5)//pause > intermessage pause (max 120ms), timer takes max 36ms to overflow and more than 170ms to overflow 5 times 
	{ IR_code.asLong &= 0x0C000000L;//set all values (except switches) to neutral, this is a failsafe in case no more commands are received (out of range)
      IR_code.asStruct.stat = 2;
	  overflow++;
	}
	CLRPIN11;
}

#if 1
//interrupt on falling edge of IR signal
ISR(IRVECT, ISR_NOBLOCK)//fast version, noblock, search tree, check only one bound
{   static unsigned old;
    static unsigned long code = 0L;
    static char ones = 0;
    static int state = 0;
    unsigned val = TIMEVAL;
    int diff  = val - old;  // diff should be positive and multiples of 500ns because timer1 is an up counter
    old = val;	  
	  
	if (diff<PULSEW0L || overflow>1) // too short or overflow (pulse+pause longer than 71ms)
	{ 	state = 0;
		code  = 0L;
		ones  = 0;
        CLRPIN11;
	}
	else 
	{ if (diff<PULSEW1U)
	  { if (diff<PULSEW0U)
	    { //code 00 received
		  state++;
		  code <<=2;
          SETPIN11;
	    }
		else
		{ //code 01 received
		  state++;
		  code <<=2;
		  code +=1;
		  ones ++;
          SETPIN11;
	    }
	  }
	  else
	  { if (diff<PULSEW2U)
	    { //code 10 received
		  state++;
		  code <<=2;
		  code +=2;
		  ones++;
          SETPIN11;
	    }
		else if (diff<PULSEW3U)
        { //code 11 received
		  state++;
		  code <<=2;
		  code +=3;
		  ones +=2;
          SETPIN11;
	    } 
		else //error, too long, go back to idle state
	    { state = 0;
		  code  = 0L;
		  ones  = 0;
          CLRPIN11;
	    }
	  }
	}
	if (state == 15)//15 pulses and 15 pauses, all bits received
	{//check frame and parity
	  code <<=2;
	  if ((code>>28) == 8 && !(ones&1) && !(code&4))
	  { IR_code.asLong = code;
	    IR_code.asStruct.stat = 2;
	  }
	  else
	  { IR_code.asStruct.stat = ones&1 ? 1 : 3; //1=PE,3=FE
	  }
	  code = 0L;
	  state = 0;
	  ones = 0;
      CLRPIN11;
	}
    overflow = 0; //reset timeout, short timeout is 33-65ms, long timeout is 128-160ms
//  CLRPIN11;
}
#else
//interrupt on falling edge of IR signal
ISR(IRVECT)
{ static unsigned old;
  static unsigned long code = 0L;
  static char ones = 0;
  static int state = 0;
//  SETPIN11;
  if (1 /*!(IRPORT & _BV(IRBIT))*/) //diff represents the pulse + pause length, ! because IR sensor inverts
  { unsigned val = TIMEVAL;
    int diff  = val - old;  // diff should be positive and multiples of 500ns because timer1 is an up counter
    old = val;	  
	  
	if (diff<PULSEW0L || overflow>1) // too short or overflow (pulse+pause longer than 71ms)
	{ 	state = 0;
		code  = 0L;
		ones  = 0;
  CLRPIN11;
	}
	else if (diff>=PULSEW0L && diff<PULSEW0U)
	{ //code 00 received
		state++;
		code <<=2;
  SETPIN11;
	} else if (diff>=PULSEW1L && diff<PULSEW1U)
	{ //code 01 received
		state++;
		code <<=2;
		code +=1;
		ones ++;
  SETPIN11;
	} else if (diff>=PULSEW2L && diff<PULSEW2U)
	{ //code 10 received
		state++;
		code <<=2;
		code +=2;
		ones++;
  SETPIN11;
	} else if (diff>=PULSEW3L && diff<PULSEW3U)
	{ //code 11 received
		state++;
		code <<=2;
		code +=3;
		ones +=2;
  SETPIN11;
	} else //error, too long, go back to idle state
	{ 	state = 0;
		code  = 0L;
		ones  = 0;
  CLRPIN11;
	}
	if (state == 15)//15 pulses and 15 pauses, all bits received
	{//check frame and parity
	  code <<=2;
	  if ((code>>28) == 8 && !(ones&1) && !(code&4))
	  { IR_code.asLong = code;
	    IR_code.asStruct.stat = 2;
	  }
	  else
	  { IR_code.asStruct.stat = ones&1 ? 1 : 3; //1=PE,3=FE
	  }
	  code = 0L;
	  state = 0;
	  ones = 0;
  CLRPIN11;
	}
    overflow = 0; //reset timeout, short timeout is 33-65ms, long timeout is 128-160ms
  }
//  CLRPIN11;
}
#endif

#endif
