#ifndef ROBO_EXT_H
#define ROBO_EXT_H

#if  F_CPU == 7372800
#define SPI_FLAGS 	 _BV(SPE) | _BV(MSTR) | _BV(SPR1) | _BV(CPOL) | _BV(CPHA) /*enable spi, master mode, clk/64 */
#elif F_CPU == 14745600
#define SPI_FLAGS	 _BV(SPE) | _BV(MSTR) | _BV(SPR1) | _BV(SPR0) | _BV(CPOL) | _BV(CPHA) /*enable spi, master mode, clk/128*/
#else
#error "define divider for cpu freq "  F_CPU
#endif

#ifdef VOLT5
#define OPENDRAIN		//simulate opendrain with pullup by tri-stating the output when high
#define EXTPULLUP		//use external pull-up resistors to 5V
#define ACKPIN	INT1
#else //3.3V
//#define OPENDRAIN		//simulate opendrain with pullup by tri-stating the output when high
#define ACKPIN	INT0
#endif

//the problem with pin change interrupt (PCINT) is that it reacts on both edges and latency makes it difficult to determine if it was rising or falling
//possibilities are INT0 (currently taken by tranceiver direction), INT1 (currently taken by IR input) and PCINT (any free I/O pin)
#if ACKPIN==INT0
#define ACKINT_vect	INT0_vect
#define ACKPORT	PIND		//the port on which ack is connected
#define ACKDIR	DDRD
#define PCMASK	EIMSK		//the register to mask the interrupt
#define	ACKBIT	2			//the bit in the port where ack is connected (int0=PD2, int1=PD3)
#define ACKMASK	0			//the bit to mask the interrupt

#elif ACKPIN==INT1
#define ACKINT_vect	INT1_vect
#define ACKPORT	PIND
#define ACKDIR	DDRD
#define PCMASK	EIMSK
#define	ACKBIT	3
#define ACKMASK	1

#else //PCINT
#define ACK		23			//the pcint number
#if ACK<8
#define ACKINT_vect	PCINT0_vect
#define ACKPORT	PINB
#define ACKDIR	DDRB
#define PCIE	PCIE0
#define PCMASK	PCMSK0
#define	ACKBIT	ACK
#elif ACK<16
#define ACKINT_vect	PCINT1_vect
#define ACKPORT	PINC
#define ACKDIR	DDRC
#define PCIE	PCIE1
#define PCMASK	PCMSK1
#define	ACKBIT	(ACK-8)
#elif ACK<24
#define ACKINT_vect	PCINT2_vect
#define ACKPORT	PIND
#define ACKDIR	DDRD
#define PCIE	PCIE2
#define PCMASK	PCMSK2
#define	ACKBIT	(ACK-16)
#else
#error port has no pcint function
#endif //ACK 
#define ACKMASK	ACKBIT
#endif //ACKPIN

#ifdef USE_ROBO_EXT
#define MASK_EXTENSION_INT()	{ SPCR = SPI_FLAGS;			  PCMASK &=~_BV(ACKMASK);}
#define UNMASK_EXTENSION_INT()	{ SPCR = SPI_FLAGS|_BV(SPIE); PCMASK |= _BV(ACKMASK);}
#else
#define MASK_EXTENSION_INT()	;
#define UNMASK_EXTENSION_INT()	;
#endif

void read_extensions();
void init_extension();
void extension_timeout();

extern uint8_t nrofext;
extern short int ack_timeout;

#endif
